package com.zanxiang.manage.service.Impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zanxiang.common.exception.BaseException;
import com.zanxiang.common.utils.StringUtils;
import com.zanxiang.common.utils.bean.BeanUtils;
import com.zanxiang.erp.security.util.SecurityUtil;
import com.zanxiang.manage.domain.dto.GameTagDTO;
import com.zanxiang.manage.domain.params.GameTagAddUpdateParam;
import com.zanxiang.manage.domain.params.GameTagListParam;
import com.zanxiang.manage.domain.vo.GameTagChoiceVO;
import com.zanxiang.manage.domain.vo.GameTagVO;
import com.zanxiang.manage.service.GameTagService;
import com.zanxiang.mybatis.entity.GameTag;
import com.zanxiang.mybatis.mapper.GameTagMapper;
import lombok.extern.slf4j.Slf4j;
import org.apache.logging.log4j.util.Strings;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @author : lingfeng
 * @time : 2022-11-16
 * @description : 游戏分类标签
 */
@Slf4j
@Service
public class GameTagServiceImpl extends ServiceImpl<GameTagMapper, GameTag> implements GameTagService {

    /**
     * 根据主键id查询分类以及子分类
     *
     * @param ids : 主键id列表
     * @return : 返回游戏分类信息
     */
    @Override
    public List<GameTagVO> listByIds(List<Long> ids) {
        if (CollectionUtils.isEmpty(ids)) {
            return Collections.emptyList();
        }
        return BeanUtils.copyList(super.listByIds(ids), GameTagVO.class);
    }

    /**
     * 标签选择列表查询
     *
     * @return : 返回所有游戏标签
     */
    @Override
    public List<GameTagChoiceVO> gameTagsChoiceList() {
        return BeanUtils.copyList(super.list(), GameTagChoiceVO.class);
    }

    /**
     * 查询游戏分类标签
     *
     * @return : 返回所有游戏分类和标签
     */
    @Override
    public Map<Long, GameTagDTO> gameTagMap() {
        List<GameTag> gameTagList = list(new LambdaQueryWrapper<GameTag>()
                .select(GameTag::getId, GameTag::getName));
        if (CollectionUtils.isEmpty(gameTagList)) {
            return Collections.emptyMap();
        }
        List<GameTagDTO> gameTagDTOList = BeanUtils.copyList(gameTagList, GameTagDTO.class);
        return gameTagDTOList.stream().collect(Collectors.toMap(GameTagDTO::getId, Function.identity()));
    }

    /**
     * 查询游戏标签列表
     *
     * @param param : 游戏标签列表查询呢参数
     * @return : 返回分页信息
     */
    @Override
    public IPage<GameTagVO> list(GameTagListParam param) {
        //模糊查询父标签列表
        List<GameTag> categoryList = new ArrayList<>();
        if (Strings.isNotBlank(param.getLabelName()) && Objects.equals(param.getIsParent(), Boolean.TRUE)) {
            categoryList = super.list(new LambdaQueryWrapper<GameTag>()
                    .like(GameTag::getName, param.getLabelName())
                    .eq(GameTag::getIsParent, Boolean.TRUE));
            //查询父标签, 但是却查不到, 直接返回空
            if (categoryList.isEmpty()) {
                return new Page<>(param.getPageNum(), param.getPageSize());
            }
        }
        List<Long> parentIdList = categoryList.stream().map(GameTag::getId).collect(Collectors.toList());
        return page(param.toPage(), new QueryWrapper<GameTag>().lambda()
                //条件不为空没勾选父标签
                .like(StringUtils.isNotBlank(param.getLabelName()) && !Objects.equals(param.getIsParent(), Boolean.TRUE),
                        GameTag::getName, param.getLabelName())
                //条件为空查询所有父标签
                .eq(StringUtils.isBlank(param.getLabelName()) && Objects.equals(param.getIsParent(), Boolean.TRUE),
                        GameTag::getIsParent, Boolean.TRUE)
                //条件不为空查询父标签
                .and(StringUtils.isNotBlank(param.getLabelName()) && Objects.equals(param.getIsParent(), Boolean.TRUE),
                        qw -> qw.in(GameTag::getParentId, parentIdList).or().like(GameTag::getName, param.getLabelName()))
                .orderByDesc(GameTag::getCreateTime))
                .convert(this::toVo);
    }

    /**
     * 游戏分类标签对象转换
     *
     * @param gameTag : 游戏分类标签信息
     * @return : 返回游戏标签
     */
    private GameTagVO toVo(GameTag gameTag) {
        GameTagVO gameTagVO = BeanUtils.copy(gameTag, GameTagVO.class);
        if (Objects.isNull(gameTagVO)) {
            return null;
        }
        if (Objects.equals(gameTag.getIsParent(), Boolean.TRUE)) {
            gameTagVO.setParentName(gameTagVO.getName());
        } else {
            GameTag parentGameTag = super.getById(gameTag.getParentId());
            if (parentGameTag != null) {
                gameTagVO.setParentName(parentGameTag.getName());
            }
        }
        return gameTagVO;
    }

    /**
     * 游戏标签分类添加或修改
     *
     * @param param : 参数
     * @return : 返回结果
     */
    @Override
    public Boolean addOrUpdate(GameTagAddUpdateParam param) {
        Long id = param.getId();
        //更新
        if (id != null) {
            GameTag gameTag = super.getById(id);
            if (gameTag == null) {
                throw new BaseException("参数错误, 游戏分类标签信息不存在");
            }
            gameTag.setIsParent(param.getParentId() == null);
            gameTag.setParentId(param.getParentId());
            gameTag.setName(param.getName());
            gameTag.setRemark(param.getRemark());
            gameTag.setUpdateTime(LocalDateTime.now());
            return super.updateById(gameTag);
        }
        //判断cp名称是否已经存在
        int count = super.count(new LambdaQueryWrapper<GameTag>()
                .eq(GameTag::getName, param.getName()));
        if (count > 0) {
            throw new BaseException("游戏分类标签名称已经存在");
        }
        //创建
        return super.save(GameTag.builder()
                .name(param.getName())
                .remark(param.getRemark())
                .isParent(param.getParentId() == null)
                .parentId(param.getParentId())
                .operatorId(SecurityUtil.getUserId())
                .operatorName(SecurityUtil.getTokenInfo().getNickname())
                .createTime(LocalDateTime.now())
                .updateTime(LocalDateTime.now())
                .build());
    }

    /**
     * 根据主键id删除
     *
     * @param id : 主键id
     * @return : 删除结果
     */
    @Override
    public Boolean deleteById(Long id) {
        return super.removeById(id);
    }
}
